@@ -1,5 +1,5 @@
-# Code by F. Murthagh, http://astro.u-strasbg.fr/~fmurtagh/mda-sw/splus
-# modified by Peter Langfelder to make it compaticle with R's standard hclust
+# Code by F. Murthagh, https://astro.u-strasbg.fr/~fmurtagh/mda-sw/splus
+# modified by Peter Langfelder to make it compatible with R's standard hclust
 
 flashClust <- function(d, method="complete",  members = NULL)
 {
@@ -43,7 +43,7 @@ hclust <- function(d, method="complete",  members = NULL)
 # -- i.e. O(n^3) -- implementations in all commercial statistical packages
 # -- as far as I am aware -- except Clustan.)  
 
- hcl <- .Fortran("hc",
+ hcl <- .Fortran(.F77_hc,
           n = as.integer(n),
           len = as.integer(len),
           method = as.integer(method),
@@ -53,8 +53,8 @@ hclust <- function(d, method="complete",  members = NULL)
           membr = as.double(members),
           nn = integer(n),
           disnn = double(n),
-          flag = logical(n),
-          diss = as.double(d))
+          diss = as.double(d), 
+          PACKAGE = "flashClust")
 
 # 2nd step: interpret the information that we now have, -- seq. of aggloms., --
 # as merge, height, and order lists.
@@ -64,13 +64,15 @@ hclust <- function(d, method="complete",  members = NULL)
  #iclass <- matrix(0.0, n, n)
  #storage.mode(iclass) <- "integer"
 
- hcass <- .Fortran("hcass2",
+ hcass <- .Fortran(.F77_hcass2,
           n = as.integer(n),
           ia = as.integer(hcl$ia),
           ib = as.integer(hcl$ib),
           order = integer(n),
           iia = integer(n),
-          iib = integer(n))
+          iib = integer(n), 
+          PACKAGE = "flashClust")
+
  merge <- cbind(hcass$iia[1:n-1],hcass$iib[1:n-1])
 
 
